@@ -49,6 +49,7 @@ namespace System.Windows.Forms {
 		private NotifyIconWindow	window;
 		private bool			systray_active;
 		private ToolTip			tooltip;
+		private bool			double_click;
 #if NET_2_0
 		private string balloon_text;
 		private string balloon_title;
@@ -75,7 +76,6 @@ namespace System.Windows.Forms {
 				SizeChanged += new EventHandler(HandleSizeChanged);
 
 				// Events that need to be sent to our parent
-				Click += new EventHandler(HandleClick);
 				DoubleClick += new EventHandler(HandleDoubleClick);
 				MouseDown +=new MouseEventHandler(HandleMouseDown);
 				MouseUp +=new MouseEventHandler(HandleMouseUp);
@@ -120,12 +120,14 @@ namespace System.Windows.Forms {
 
 							case Msg.WM_LBUTTONUP: {
 								owner.OnMouseUp (new MouseEventArgs(MouseButtons.Left, 1, Control.MousePosition.X, Control.MousePosition.Y, 0));
-								owner.OnClick (EventArgs.Empty);
 								return;
 							}
 
 							case Msg.WM_LBUTTONDBLCLK: {
 								owner.OnDoubleClick (EventArgs.Empty);
+#if NET_2_0
+								owner.OnMouseDoubleClick (new MouseEventArgs (MouseButtons.Left, 2, Control.MousePosition.X, Control.MousePosition.Y, 0));
+#endif
 								return;
 							}
 
@@ -141,12 +143,14 @@ namespace System.Windows.Forms {
 
 							case Msg.WM_RBUTTONUP: {
 								owner.OnMouseUp (new MouseEventArgs(MouseButtons.Right, 1, Control.MousePosition.X, Control.MousePosition.Y, 0));
-								owner.OnClick (EventArgs.Empty);
 								return;
 							}
 
 							case Msg.WM_RBUTTONDBLCLK: {
 								owner.OnDoubleClick (EventArgs.Empty);
+#if NET_2_0
+								owner.OnMouseDoubleClick (new MouseEventArgs (MouseButtons.Left, 2, Control.MousePosition.X, Control.MousePosition.Y, 0));
+#endif
 								return;
 							}
 #if NET_2_0							
@@ -193,7 +197,9 @@ namespace System.Windows.Forms {
 
 			internal override void OnPaintInternal (PaintEventArgs e) {
 				if (owner.icon != null) {
-					e.Graphics.FillRectangle(ThemeEngine.Current.ResPool.GetSolidBrush(SystemColors.Window), rect);
+					// At least in Gnome, the background of the panel is the same as the Menu, so we go for it
+					// instead of (most of the time) plain white.
+					e.Graphics.FillRectangle(ThemeEngine.Current.ResPool.GetSolidBrush(SystemColors.Menu), rect);
 					e.Graphics.DrawImage(owner.icon_bitmap,
 							     rect,
 							     new Rectangle (0, 0, owner.icon_bitmap.Width, owner.icon_bitmap.Height),
@@ -206,19 +212,16 @@ namespace System.Windows.Forms {
 				base.RecreateHandle ();
 			}
 
-
 			private void HandleSizeChanged(object sender, EventArgs e) {
 				owner.Recalculate ();
 			}
 
-			private void HandleClick (object sender, EventArgs e)
-			{
-				owner.OnClick (e);
-			}
-
 			private void HandleDoubleClick (object sender, EventArgs e)
 			{
 				owner.OnDoubleClick (e);
+#if NET_2_0
+				owner.OnMouseDoubleClick (new MouseEventArgs (MouseButtons.Left, 2, Control.MousePosition.X, Control.MousePosition.Y, 0));
+#endif
 			}
 
 			private void HandleMouseDown (object sender, MouseEventArgs e)
@@ -262,6 +265,12 @@ namespace System.Windows.Forms {
 				timer.Enabled = false;
 				timer.Tick += new EventHandler (HandleTimer);
 			}
+
+			public IntPtr OwnerHandle {
+				get {
+					return owner;
+				}
+			}
 			
 			protected override void Dispose (bool disposing)
 			{
@@ -414,9 +423,9 @@ namespace System.Windows.Forms {
 			ShowBalloonTip(timeout, balloon_title, balloon_text, balloon_icon);
 		}
 
-		public void ShowBalloonTip(int timeout, string title, string text, ToolTipIcon icon)
+		public void ShowBalloonTip(int timeout, string tipTitle, string tipText, ToolTipIcon tipIcon)
 		{
-			XplatUI.SystrayBalloon(window.Handle, timeout, title, text, icon);
+			XplatUI.SystrayBalloon(window.Handle, timeout, tipTitle, tipText, tipIcon);
 		}
 #endif
 		#endregion Public Methods
@@ -453,11 +462,28 @@ namespace System.Windows.Forms {
 
 		private void OnDoubleClick (EventArgs e)
 		{
+			double_click = true;
 			EventHandler eh = (EventHandler)(Events [DoubleClickEvent]);
 			if (eh != null)
 				eh (this, e);
 		}
 
+#if NET_2_0
+		private void OnMouseClick (MouseEventArgs e)
+		{
+			MouseEventHandler eh = (MouseEventHandler)(Events[MouseClickEvent]);
+			if (eh != null)
+				eh (this, e);
+		}
+		
+		private void OnMouseDoubleClick (MouseEventArgs e)
+		{
+			MouseEventHandler eh = (MouseEventHandler)(Events[MouseDoubleClickEvent]);
+			if (eh != null)
+				eh (this, e);
+		}
+#endif
+
 		private void OnMouseDown (MouseEventArgs e)
 		{
 			MouseEventHandler eh = (MouseEventHandler)(Events [MouseDownEvent]);
@@ -467,16 +493,30 @@ namespace System.Windows.Forms {
 
 		private void OnMouseUp (MouseEventArgs e)
 		{
-			if ((e.Button & MouseButtons.Right) == MouseButtons.Right && context_menu != null)
-				context_menu.Show (window, new Point(e.X, e.Y));
+			if ((e.Button & MouseButtons.Right) == MouseButtons.Right) {
+				if (context_menu != null) {
+					XplatUI.SetForegroundWindow (window.Handle);
+					context_menu.Show (window, new Point(e.X, e.Y));
+				} 
 #if NET_2_0
-			else if ((e.Button & MouseButtons.Right) == MouseButtons.Right && context_menu_strip != null)
-				context_menu_strip.Show (window, new Point (e.X, e.Y), ToolStripDropDownDirection.AboveLeft);
+				else if (context_menu_strip != null) {
+					XplatUI.SetForegroundWindow (window.Handle);
+					context_menu_strip.Show (window, new Point (e.X, e.Y), ToolStripDropDownDirection.AboveLeft);
+				}
 #endif
-			
+			}
+
 			MouseEventHandler eh = (MouseEventHandler)(Events [MouseUpEvent]);
 			if (eh != null)
 				eh (this, e);
+
+			if (!double_click) {
+				OnClick (EventArgs.Empty);
+#if NET_2_0
+				OnMouseClick (e);
+#endif
+				double_click = false;
+			}
 		}
 
 		private void OnMouseMove (MouseEventArgs e)
@@ -490,19 +530,25 @@ namespace System.Windows.Forms {
 		{
 			window.CalculateIconRect ();
 
-			if (systray_active)
-				UpdateSystray ();
+			if (!Visible || (text == string.Empty && icon == null)) {
+				HideSystray ();
+			} else {
+
+				if (systray_active)
+					UpdateSystray ();
+				else
+					ShowSystray ();
+			}
 		}
 
 		private void ShowSystray()
 		{
-			systray_active = true;
-
 			if (icon == null)
 				return;
 
 			icon_bitmap = icon.ToBitmap();
 
+			systray_active = true;
 			XplatUI.SystrayAdd(window.Handle, text, icon, out tooltip);
 		}
 
@@ -526,13 +572,14 @@ namespace System.Windows.Forms {
 				icon_bitmap = icon.ToBitmap();
 			}
 
-			XplatUI.SystrayChange(window.Handle, text, icon, ref tooltip);
 			window.Invalidate();
+			XplatUI.SystrayChange(window.Handle, text, icon, ref tooltip);
 		}
 		#endregion	// Private Methods
 
 		#region Public Instance Properties
 #if NET_2_0
+		[DefaultValue ("None")]
 		public ToolTipIcon BalloonTipIcon {
 			get { return this.balloon_icon; }
 			set {
@@ -544,6 +591,9 @@ namespace System.Windows.Forms {
 		}
 
 		[Localizable(true)]
+		[DefaultValue ("")]
+		[Editor ("System.ComponentModel.Design.MultilineStringEditor, " + Consts.AssemblySystem_Design,
+			 "System.Drawing.Design.UITypeEditor, " + Consts.AssemblySystem_Drawing)]
 		public string BalloonTipText {
 			get { return this.balloon_text; }
 			set {
@@ -555,6 +605,7 @@ namespace System.Windows.Forms {
 		}
  		
 		[Localizable(true)]
+		[DefaultValue ("")]
 		public string BalloonTipTitle {
 			get { return this.balloon_title; }
 			set {
@@ -606,12 +657,7 @@ namespace System.Windows.Forms {
 			set {
 				if (icon != value) {
 					icon = value;
-					if (text == string.Empty && icon == null) {
-						HideSystray ();
-					}
-					else {
-						Recalculate ();
-					}
+					Recalculate ();
 				}
 			}
 		}
@@ -642,11 +688,7 @@ namespace System.Windows.Forms {
 						throw new ArgumentException("ToolTip length must be less than 64 characters long", "Text");
 					}
 					text = value;
-					if (text == string.Empty && icon == null) {
-						HideSystray();
-					} else {
-						Recalculate ();
-					}
+					Recalculate ();
 				}
 			}
 		}
@@ -703,6 +745,8 @@ namespace System.Windows.Forms {
 		static object BalloonTipClickedEvent = new object ();
 		static object BalloonTipClosedEvent = new object ();
 		static object BalloonTipShownEvent = new object ();
+		static object MouseClickEvent = new object ();
+		static object MouseDoubleClickEvent = new object ();
 
 		[MWFCategory("Action")]
 		public event EventHandler BalloonTipClicked {
@@ -721,6 +765,18 @@ namespace System.Windows.Forms {
 			add { Events.AddHandler (BalloonTipShownEvent, value); }
 			remove { Events.RemoveHandler (BalloonTipShownEvent, value); }
 		}
+
+		[MWFCategory("Action")]
+		public event MouseEventHandler MouseClick {
+			add { Events.AddHandler (MouseClickEvent, value); }
+			remove { Events.RemoveHandler (MouseClickEvent, value); }
+		}
+
+		[MWFCategory ("Action")]
+		public event MouseEventHandler MouseDoubleClick {
+			add { Events.AddHandler (MouseDoubleClickEvent, value); }
+			remove { Events.RemoveHandler (MouseDoubleClickEvent, value); }
+		}
 #endif
 
 #if NET_2_0
