@@ -267,6 +267,12 @@ namespace Mono.Unix.Native {
 	}
 
 
+	public enum SignalAction {
+		Default,
+		Ignore,
+		Error
+	}
+
 	//
 	// Right now using this attribute gives an assert because it
 	// isn't implemented.
@@ -454,6 +460,8 @@ namespace Mono.Unix.Native {
 		private static extern IntPtr sys_signal (int signum, IntPtr handler);
 
 		[CLSCompliant (false)]
+		[Obsolete ("This is not safe; " + 
+				"use Mono.Unix.UnixSignal for signal delivery or SetSignalAction()")]
 		public static SignalHandler signal (Signum signum, SignalHandler handler)
 		{
 			int _sig = NativeConvert.FromSignum (signum);
@@ -494,14 +502,50 @@ namespace Mono.Unix.Native {
 #endif
 		}
 
+		public static int SetSignalAction (Signum signal, SignalAction action)
+		{
+			return SetSignalAction (NativeConvert.FromSignum (signal), action);
+		}
+
+		public static int SetSignalAction (RealTimeSignum rts, SignalAction action)
+		{
+			return SetSignalAction (NativeConvert.FromRealTimeSignum (rts), action);
+		}
+		
+		private static int SetSignalAction (int signum, SignalAction action)
+		{
+			IntPtr handler = IntPtr.Zero;
+			switch (action) {
+				case SignalAction.Default:
+					handler = _SIG_DFL;
+					break;
+				case SignalAction.Ignore:
+					handler = _SIG_IGN;
+					break;
+				case SignalAction.Error:
+					handler = _SIG_ERR;
+					break;
+				default:
+					throw new ArgumentException ("Invalid action value.", "action");
+			}
+			IntPtr r = sys_signal (signum, handler);
+			if (r == _SIG_ERR)
+				return -1;
+			return 0;
+		}
+
 		[DllImport (LIBC, CallingConvention=CallingConvention.Cdecl, EntryPoint="raise")]
 		private static extern int sys_raise (int sig);
 
 		[CLSCompliant (false)]
 		public static int raise (Signum sig)
 		{
-			int _sig = NativeConvert.FromSignum (sig);
-			return sys_raise (_sig);
+			return sys_raise (NativeConvert.FromSignum (sig));
+		}
+
+		public static int raise (RealTimeSignum rts)
+		{
+			return sys_raise (NativeConvert.FromRealTimeSignum (rts));
 		}
 
 		//
@@ -926,7 +970,12 @@ namespace Mono.Unix.Native {
 
 		[DllImport (MPH, CallingConvention=CallingConvention.Cdecl, 
 				SetLastError=true, EntryPoint="Mono_Posix_Stdlib_perror")]
-		public static extern int perror (string s);
+		private static extern int perror (string s, int err);
+
+		public static int perror (string s)
+		{
+			return perror (s, Marshal.GetLastWin32Error ());
+		}
 
 		//
 		// <stdlib.h>
