@@ -258,7 +258,7 @@ namespace System.Data {
 			DataRowChangeEventArgs e = new DataRowChangeEventArgs (dr, action);
 			OnRowChanging (e);
 		}
-
+		
 		/// <summary>
 		/// Gets the collection of child relations for this DataTable.
 		/// </summary>
@@ -773,18 +773,10 @@ namespace System.Data {
 		/// </summary>
 		public void Clear ()
 		{
-			DataTableClearing ();
 			// Foriegn key constraints are checked in _rows.Clear method
 			_rows.Clear ();
-			foreach (Index index in Indexes)
-				index.Reset ();
-			DataTableCleared ();
 		}
 
-		// defined in the NET_2_0 profile
-		partial void DataTableClearing ();
-		partial void DataTableCleared ();
-
 		/// <summary>
 		/// Clones the structure of the DataTable, including
 		///  all DataTable schemas and constraints.
@@ -888,6 +880,8 @@ namespace System.Data {
 				if (isEmpty || !Copy.Columns.Contains (column.ColumnName))
 					Copy.Columns.Add (column.Clone ());
 			}
+			foreach (DataColumn column in Copy.Columns)
+				column.CompileExpression ();
 
 			CopyConstraints (Copy);
 			// add primary key to the copy
@@ -2757,8 +2751,8 @@ namespace System.Data {
 				if (rowsCount == 0)
 					continue;
 				BitArray nullBits = new BitArray (rowsCount);
-				Array recordArray = Array.CreateInstance (Rows[0][j].GetType (), recordsCount);
 				DataColumn column = Columns [j];
+				Array recordArray = Array.CreateInstance (column.DataType, recordsCount);
 				for (int k = 0, l = 0; k < Rows.Count; k++, l++) {
 					DataRowVersion version;
 					DataRow dr = Rows[k];
@@ -2987,7 +2981,7 @@ namespace System.Data {
 				TableCleared (this, e);
 		}
 
-		partial void DataTableCleared ()
+		internal void DataTableCleared ()
 		{
 			OnTableCleared (new DataTableClearEventArgs (this));
 		}
@@ -2998,7 +2992,7 @@ namespace System.Data {
 				TableClearing (this, e);
 		}
 
-		partial void DataTableClearing ()
+		internal void DataTableClearing ()
 		{
 			OnTableClearing (new DataTableClearEventArgs (this));
 		}
