@@ -3,7 +3,7 @@
 //
 // Authors:
 //	Sureshkumar T (tsureshkumar@novell.com)
-//	Marek Safar (marek.safar@seznam.cz) (stubs)
+//	Marek Safar (marek.safar@gmail.com)
 //	Ankit Jain (radical@corewars.org)
 //	David Waite (mass@akuma.org)
 //	Juraj Skripsky (js@hotfeet.ch)
@@ -33,7 +33,6 @@
 // WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 //
 
-#if NET_2_0
 
 using System;
 using System.Collections;
@@ -41,10 +40,23 @@ using System.Collections.Generic;
 using System.Runtime.Serialization;
 using System.Security.Permissions;
 using System.Runtime.InteropServices;
+using System.Diagnostics;
 
 namespace System.Collections.Generic {
+
+	/* 
+	 * Declare this outside the main class so it doesn't have to be inflated for each
+	 * instantiation of Dictionary.
+	 */
+	internal struct Link {
+		public int HashCode;
+		public int Next;
+	}
+
 	[ComVisible(false)]
 	[Serializable]
+	[DebuggerDisplay ("Count={Count}")]
+	[DebuggerTypeProxy (typeof (CollectionDebuggerView<,>))]
 	public class Dictionary<TKey, TValue> : IDictionary<TKey, TValue>,
 		IDictionary,
 		ICollection,
@@ -73,11 +85,6 @@ namespace System.Collections.Generic {
 		const float DEFAULT_LOAD_FACTOR = (90f / 100);
 		const int NO_SLOT = -1;
 		const int HASH_FLAG = -2147483648;
-		
-		private struct Link {
-			public int HashCode;
-			public int Next;
-		}
 
 		// The hash table contains indices into the linkSlots array
 		int [] table;
@@ -280,8 +287,8 @@ namespace System.Collections.Generic {
 			if (threshold == 0 && table.Length > 0)
 				threshold = 1;
 		}
-		
-		void CopyTo (KeyValuePair<TKey, TValue> [] array, int index)
+
+		void CopyToCheck (Array array, int index)
 		{
 			if (array == null)
 				throw new ArgumentNullException ("array");
@@ -292,13 +299,76 @@ namespace System.Collections.Generic {
 				throw new ArgumentException ("index larger than largest valid index of array");
 			if (array.Length - index < Count)
 				throw new ArgumentException ("Destination array cannot hold the requested elements!");
+		}
 
+		void CopyKeys (TKey[] array, int index)
+		{
+			for (int i = 0; i < touchedSlots; i++) {
+				if ((linkSlots [i].HashCode & HASH_FLAG) != 0)
+					array [index++] = keySlots [i];
+			}
+		}
+
+		void CopyValues (TValue[] array, int index)
+		{
+			for (int i = 0; i < touchedSlots; i++) {
+				if ((linkSlots [i].HashCode & HASH_FLAG) != 0)
+					array [index++] = valueSlots [i];
+			}
+		}
+
+		delegate TRet Transform<TRet> (TKey key, TValue value);
+
+
+		static KeyValuePair<TKey, TValue> make_pair (TKey key, TValue value)
+		{
+			return new KeyValuePair<TKey, TValue> (key, value);
+		}
+
+		static TKey pick_key (TKey key, TValue value)
+		{
+			return key;
+		}
+
+		static TValue pick_value (TKey key, TValue value)
+		{
+			return value;
+		}
+
+		void CopyTo (KeyValuePair<TKey, TValue> [] array, int index)
+		{
+			CopyToCheck (array, index);
 			for (int i = 0; i < touchedSlots; i++) {
 				if ((linkSlots [i].HashCode & HASH_FLAG) != 0)
 					array [index++] = new KeyValuePair<TKey, TValue> (keySlots [i], valueSlots [i]);
 			}
 		}
-		
+
+		void Do_ICollectionCopyTo<TRet> (Array array, int index, Transform<TRet> transform)
+		{
+			Type src = typeof (TRet);
+			Type tgt = array.GetType ().GetElementType ();
+
+			try {
+				if ((src.IsPrimitive || tgt.IsPrimitive) && !tgt.IsAssignableFrom (src))
+					throw new Exception (); // we don't care.  it'll get transformed to an ArgumentException below
+
+#if BOOTSTRAP_BASIC
+				// BOOTSTRAP: gmcs 2.4.x seems to have trouble compiling the alternative
+				throw new Exception ();
+#else
+				object[] dest = (object[])array;
+				for (int i = 0; i < touchedSlots; i++) {
+					if ((linkSlots [i].HashCode & HASH_FLAG) != 0)
+						dest [index++] = transform (keySlots [i], valueSlots [i]);
+				}
+#endif
+
+			} catch (Exception e) {
+				throw new ArgumentException ("Cannot copy source collection elements to destination array", "array", e);
+			}
+		}
+
 		private void Resize ()
 		{
 			// From the SDK docs:
@@ -446,11 +516,13 @@ namespace System.Collections.Generic {
 
 			info.AddValue ("Version", generation);
 			info.AddValue ("Comparer", hcp);
-			KeyValuePair<TKey, TValue> [] data = null;
-			if (count > 0) {
-				data = new KeyValuePair<TKey,TValue> [count];
+			// MS.NET expects either *no* KeyValuePairs field (when count = 0)
+			// or a non-null KeyValuePairs field. We don't omit the field to
+			// remain compatible with older monos, but we also doesn't serialize
+			// it as null to make MS.NET happy.
+			KeyValuePair<TKey, TValue> [] data = new KeyValuePair<TKey,TValue> [count];
+			if (count > 0)
 				CopyTo (data, 0);
-			}
 			info.AddValue ("HashSize", table.Length);
 			info.AddValue ("KeyValuePairs", data);
 		}
@@ -460,14 +532,34 @@ namespace System.Collections.Generic {
 			if (serialization_info == null)
 				return;
 
-			generation = serialization_info.GetInt32 ("Version");
-			hcp = (IEqualityComparer<TKey>) serialization_info.GetValue ("Comparer", typeof (IEqualityComparer<TKey>));
+			int hashSize = 0;
+			KeyValuePair<TKey, TValue> [] data = null;
 
-			int hashSize = serialization_info.GetInt32 ("HashSize");
-			KeyValuePair<TKey, TValue> [] data =
-				(KeyValuePair<TKey, TValue> [])
-				serialization_info.GetValue ("KeyValuePairs", typeof (KeyValuePair<TKey, TValue> []));
+			// We must use the enumerator because MS.NET doesn't
+			// serialize "KeyValuePairs" for count = 0.
+			SerializationInfoEnumerator e = serialization_info.GetEnumerator ();
+			while (e.MoveNext ()) {
+				switch (e.Name) {
+				case "Version":
+					generation = (int) e.Value;
+					break;
 
+				case "Comparer":
+					hcp = (IEqualityComparer<TKey>) e.Value;
+					break;
+
+				case "HashSize":
+					hashSize = (int) e.Value;
+					break;
+
+				case "KeyValuePairs":
+					data = (KeyValuePair<TKey, TValue> []) e.Value;
+					break;
+				}
+			}
+
+			if (hcp == null)
+				hcp = EqualityComparer<TKey>.Default;
 			if (hashSize < INITIAL_SIZE)
 				hashSize = INITIAL_SIZE;
 			InitArrays (hashSize);
@@ -684,22 +776,13 @@ namespace System.Collections.Generic {
 
 		void ICollection.CopyTo (Array array, int index)
 		{
-			if (array == null)
-				throw new ArgumentNullException ("array");
-			if (index < 0)
-				throw new ArgumentOutOfRangeException ("index");
-			// we want no exception for index==array.Length && Count == 0
-			if (index > array.Length)
-				throw new ArgumentException ("index larger than largest valid index of array");
-			if (array.Length - index < count)
-				throw new ArgumentException ("Destination array cannot hold the requested elements!");
-
 			KeyValuePair<TKey, TValue> [] pairs = array as KeyValuePair<TKey, TValue> [];
 			if (pairs != null) {
 				this.CopyTo (pairs, index);
 				return;
 			}
 
+			CopyToCheck (array, index);
 			DictionaryEntry [] entries = array as DictionaryEntry [];
 			if (entries != null) {
 				for (int i = 0; i < touchedSlots; i++) {
@@ -709,16 +792,7 @@ namespace System.Collections.Generic {
 				return;
 			}
 
-			object [] objects = array as object [];
-			if (objects != null && objects.GetType () == typeof (object [])) {
-				for (int i = 0; i < touchedSlots; i++) {
-					if ((linkSlots [i].HashCode & HASH_FLAG) != 0)
-						objects [index++] = new KeyValuePair<TKey, TValue> (keySlots [i], valueSlots [i]);
-				}
-				return;
-			}
-
-			throw new ArgumentException ("Invalid array type");
+			Do_ICollectionCopyTo<KeyValuePair<TKey, TValue>> (array, index, make_pair);
 		}
 
 		IEnumerator IEnumerable.GetEnumerator ()
@@ -789,54 +863,64 @@ namespace System.Collections.Generic {
 			IDisposable, IDictionaryEnumerator, IEnumerator
 		{
 			Dictionary<TKey, TValue> dictionary;
-			int cur;
+			int next;
 			int stamp;
-			const int NOT_STARTED = -1; // must be -1
+
+			internal KeyValuePair<TKey, TValue> current;
 
 			internal Enumerator (Dictionary<TKey, TValue> dictionary)
+				: this ()
 			{
 				this.dictionary = dictionary;
 				stamp = dictionary.generation;
-
-				cur = NOT_STARTED;
 			}
 
 			public bool MoveNext ()
 			{
 				VerifyState ();
-				while (cur < dictionary.touchedSlots) {
-					if ((dictionary.linkSlots [++cur].HashCode & HASH_FLAG) != 0)
+
+				if (next < 0)
+					return false;
+
+				while (next < dictionary.touchedSlots) {
+					int cur = next++;
+					if ((dictionary.linkSlots [cur].HashCode & HASH_FLAG) != 0) {
+						current = new KeyValuePair <TKey, TValue> (
+							dictionary.keySlots [cur],
+							dictionary.valueSlots [cur]
+							);
 						return true;
+					}
 				}
+
+				next = -1;
 				return false;
 			}
 
+			// No error checking happens.  Usually, Current is immediately preceded by a MoveNext(), so it's wasteful to check again
 			public KeyValuePair<TKey, TValue> Current {
-				get { 
-					VerifyCurrent (); 
-					return new KeyValuePair <TKey, TValue> (
-						dictionary.keySlots [cur],
-						dictionary.valueSlots [cur]
-					);
-				}
+				get { return current; }
 			}
 			
 			internal TKey CurrentKey {
 				get {
 					VerifyCurrent ();
-					return dictionary.keySlots [cur];
+					return current.Key;
 				}
 			}
 			
 			internal TValue CurrentValue {
 				get {
 					VerifyCurrent ();
-					return dictionary.valueSlots [cur];
+					return current.Value;
 				}
 			}
 
 			object IEnumerator.Current {
-				get { return Current; }
+				get {
+					VerifyCurrent ();
+					return current;
+				}
 			}
 
 			void IEnumerator.Reset ()
@@ -846,31 +930,23 @@ namespace System.Collections.Generic {
 
 			internal void Reset ()
 			{
-				cur = NOT_STARTED;
+				VerifyState ();
+				next = 0;
 			}
 
 			DictionaryEntry IDictionaryEnumerator.Entry {
 				get {
 					VerifyCurrent ();
-					return new DictionaryEntry (
-						dictionary.keySlots [cur],
-						dictionary.valueSlots [cur]
-					);
+					return new DictionaryEntry (current.Key, current.Value);
 				}
 			}
 
 			object IDictionaryEnumerator.Key {
-				get {
-					VerifyCurrent();
-					return dictionary.keySlots [cur];
-				}
+				get { return CurrentKey; }
 			}
 
 			object IDictionaryEnumerator.Value {
-				get {
-					VerifyCurrent();
-					return dictionary.valueSlots [cur];
-				}
+				get { return CurrentValue; }
 			}
 
 			void VerifyState ()
@@ -884,7 +960,7 @@ namespace System.Collections.Generic {
 			void VerifyCurrent ()
 			{
 				VerifyState ();
-				if (cur == NOT_STARTED || cur >= dictionary.touchedSlots)
+				if (next <= 0)
 					throw new InvalidOperationException ("Current is not valid");
 			}
 
@@ -896,6 +972,8 @@ namespace System.Collections.Generic {
 
 		// This collection is a read only collection
 		[Serializable]
+		[DebuggerDisplay ("Count={Count}")]
+		[DebuggerTypeProxy (typeof (CollectionDebuggerView<,>))]		
 		public sealed class KeyCollection : ICollection<TKey>, IEnumerable<TKey>, ICollection, IEnumerable {
 			Dictionary<TKey, TValue> dictionary;
 
@@ -906,26 +984,11 @@ namespace System.Collections.Generic {
 				this.dictionary = dictionary;
 			}
 
-			void CopyToCheck (IList array, int index)
-			{
-				if (array == null)
-					throw new ArgumentNullException ("array");
-				if (index < 0)
-					throw new ArgumentOutOfRangeException ("index");
-				// we want no exception for index==array.Length && dictionary.Count == 0
-				if (index > array.Count)
-					throw new ArgumentException ("index larger than largest valid index of array");
-				if (array.Count - index < dictionary.Count)
-					throw new ArgumentException ("Destination array cannot hold the requested elements!");
-			}
 
 			public void CopyTo (TKey [] array, int index)
 			{
-				CopyToCheck ((IList)array, index);
-				for (int i = 0; i < dictionary.touchedSlots; i++) {
-					if ((dictionary.linkSlots [i].HashCode & HASH_FLAG) != 0)
-						array [index++] = dictionary.keySlots [i];
-				}
+				dictionary.CopyToCheck (array, index);
+				dictionary.CopyKeys (array, index);
 			}
 
 			public Enumerator GetEnumerator ()
@@ -960,12 +1023,14 @@ namespace System.Collections.Generic {
 
 			void ICollection.CopyTo (Array array, int index)
 			{
-				IList list = array;
-				CopyToCheck (list, index);
-				for (int i = 0; i < dictionary.touchedSlots; i++) {
-					if ((dictionary.linkSlots [i].HashCode & HASH_FLAG) != 0)
-						list [index++] = dictionary.keySlots [i];
+				var target = array as TKey [];
+				if (target != null) {
+					CopyTo (target, index);
+					return;
 				}
+
+				dictionary.CopyToCheck (array, index);
+				dictionary.Do_ICollectionCopyTo<TKey> (array, index, pick_key);
 			}
 
 			IEnumerator IEnumerable.GetEnumerator ()
@@ -1009,7 +1074,7 @@ namespace System.Collections.Generic {
 				}
 
 				public TKey Current {
-					get { return host_enumerator.CurrentKey; }
+					get { return host_enumerator.current.Key; }
 				}
 
 				object IEnumerator.Current {
@@ -1025,6 +1090,8 @@ namespace System.Collections.Generic {
 
 		// This collection is a read only collection
 		[Serializable]
+		[DebuggerDisplay ("Count={Count}")]
+		[DebuggerTypeProxy (typeof (CollectionDebuggerView<,>))]		
 		public sealed class ValueCollection : ICollection<TValue>, IEnumerable<TValue>, ICollection, IEnumerable {
 			Dictionary<TKey, TValue> dictionary;
 
@@ -1037,20 +1104,8 @@ namespace System.Collections.Generic {
 
 			public void CopyTo (TValue [] array, int index)
 			{
-				if (array == null)
-					throw new ArgumentNullException ("array");
-				if (index < 0)
-					throw new ArgumentOutOfRangeException ("index");
-				// we want no exception for index==array.Length && dictionary.Count == 0
-				if (index > array.Length)
-					throw new ArgumentException ("index larger than largest valid index of array");
-				if (array.Length - index < dictionary.Count)
-					throw new ArgumentException ("Destination array cannot hold the requested elements!");
-
-				for (int i = 0; i < dictionary.touchedSlots; i++) {
-					if ((dictionary.linkSlots [i].HashCode & HASH_FLAG) != 0)
-						array [index++] = dictionary.valueSlots [i];
-				}
+				dictionary.CopyToCheck (array, index);
+				dictionary.CopyValues (array, index);
 			}
 
 			public Enumerator GetEnumerator ()
@@ -1085,7 +1140,14 @@ namespace System.Collections.Generic {
 
 			void ICollection.CopyTo (Array array, int index)
 			{
-				CopyTo ((TValue []) array, index);
+				var target = array as TValue [];
+				if (target != null) {
+					CopyTo (target, index);
+					return;
+				}
+
+				dictionary.CopyToCheck (array, index);
+				dictionary.Do_ICollectionCopyTo<TValue> (array, index, pick_value);
 			}
 
 			IEnumerator IEnumerable.GetEnumerator ()
@@ -1120,7 +1182,7 @@ namespace System.Collections.Generic {
 
 				public void Dispose ()
 				{
-					host_enumerator.Dispose();
+					host_enumerator.Dispose ();
 				}
 
 				public bool MoveNext ()
@@ -1129,7 +1191,7 @@ namespace System.Collections.Generic {
 				}
 
 				public TValue Current {
-					get { return host_enumerator.CurrentValue; }
+					get { return host_enumerator.current.Value; }
 				}
 
 				object IEnumerator.Current {
@@ -1144,4 +1206,3 @@ namespace System.Collections.Generic {
 		}
 	}
 }
-#endif
