@@ -390,11 +390,11 @@ namespace System {
 			                                     culture, activationAttributes, null);
 		}
 
-		public object CreateInstanceFromAndUnwrap (string assemblyName, string typeName, bool ignoreCase,
+		public object CreateInstanceFromAndUnwrap (string assemblyFile, string typeName, bool ignoreCase,
 		                                           BindingFlags bindingAttr, Binder binder, object[] args,
 		                                           CultureInfo culture, object[] activationAttributes)
 		{
-			ObjectHandle oh = CreateInstanceFrom (assemblyName, typeName, ignoreCase, bindingAttr, binder, args,
+			ObjectHandle oh = CreateInstanceFrom (assemblyFile, typeName, ignoreCase, bindingAttr, binder, args,
 				culture, activationAttributes);
 
 			return (oh != null) ? oh.Unwrap () : null;
@@ -739,14 +739,14 @@ namespace System {
 			if (assemblyRef.Name != aname.Name)
 				throw new FileNotFoundException (null, assemblyRef.Name);
 
-			if (assemblyRef.Version != new Version () && assemblyRef.Version != aname.Version)
+			if (assemblyRef.Version != null && assemblyRef.Version != new Version (0, 0, 0, 0) && assemblyRef.Version != aname.Version)
 				throw new FileNotFoundException (null, assemblyRef.Name);
 
 			if (assemblyRef.CultureInfo != null && assemblyRef.CultureInfo.Equals (aname))
 				throw new FileNotFoundException (null, assemblyRef.Name);
 
 			byte [] pt = assemblyRef.GetPublicKeyToken ();
-			if (pt != null) {
+			if (pt != null && pt.Length != 0) {
 				byte [] loaded_pt = aname.GetPublicKeyToken ();
 				if (loaded_pt == null || (pt.Length != loaded_pt.Length))
 					throw new FileNotFoundException (null, assemblyRef.Name);
@@ -1316,6 +1316,25 @@ namespace System {
 			}
 		}
 
+		internal Assembly DoResourceResolve (string name, Assembly requesting) {
+			if (ResourceResolve == null)
+				return null;
+
+			Delegate[] invocation_list = ResourceResolve.GetInvocationList ();
+
+			foreach (Delegate eh in invocation_list) {
+				ResolveEventHandler handler = (ResolveEventHandler) eh;
+#if NET_4_0
+				Assembly assembly = handler (this, new ResolveEventArgs (name, requesting));
+#else
+				Assembly assembly = handler (this, new ResolveEventArgs (name));
+#endif
+				if (assembly != null)
+					return assembly;
+			}
+			return null;
+		}
+
 		private void DoDomainUnload ()
 		{
 			if (DomainUnload != null)
@@ -1374,7 +1393,7 @@ namespace System {
 		[method: SecurityPermission (SecurityAction.LinkDemand, ControlAppDomain = true)]
 		public event UnhandledExceptionEventHandler UnhandledException;
 
-#if NET_4_0 || BOOTSTRAP_NET_4_0
+#if NET_4_0
 		[MonoTODO]
 		public bool IsHomogenous {
 			get { return true; }
@@ -1536,14 +1555,23 @@ namespace System {
 		}
 #endif
 
-#if NET_4_0 || MOONLIGHT
-		[MonoTODO ("Currently always returns false")]
+#if NET_4_0 || MOONLIGHT || MOBILE
+		List<string> compatibility_switch;
+
 		public bool? IsCompatibilitySwitchSet (string value)
 		{
 			if (value == null)
 				throw new ArgumentNullException ("value");
+
 			// default (at least for SL4) is to return false for unknown values (can't get a null out of it)
-			return false;
+			return ((compatibility_switch != null) && compatibility_switch.Contains (value));
+		}
+
+		internal void SetCompatibilitySwitch (string value)
+		{
+			if (compatibility_switch == null)
+				compatibility_switch = new List<string> ();
+			compatibility_switch.Add (value);
 		}
 
 		[MonoTODO ("Currently always returns false")]
