@@ -216,14 +216,15 @@ namespace MonoTests.System.Reflection
 			Assert.IsFalse (corlib_test.GlobalAssemblyCache, "GlobalAssemblyCache");
 
 			Assert.IsTrue (corlib_test.GetReferencedAssemblies ().Length > 0, "GetReferencedAssemblies");
-#if NET_2_0
 			Assert.AreEqual (0, corlib_test.HostContext, "HostContext");
+#if NET_4_0
+			Assert.AreEqual ("v4.0.30319", corlib_test.ImageRuntimeVersion, "ImageRuntimeVersion");
+#else
 			Assert.AreEqual ("v2.0.50727", corlib_test.ImageRuntimeVersion, "ImageRuntimeVersion");
+#endif
+
 			Assert.IsNotNull (corlib_test.ManifestModule, "ManifestModule");
 			Assert.IsFalse (corlib_test.ReflectionOnly, "ReflectionOnly");
-#elif NET_1_1
-			Assert.AreEqual ("v1.1.4322", corlib_test.ImageRuntimeVersion, "ImageRuntimeVersion");
-#endif
 		}
 #endif
 
@@ -420,12 +421,12 @@ namespace MonoTests.System.Reflection
 		[Test]
 		public void LoadWithPartialName ()
 		{
-			string [] names = { "corlib_test_default", "corlib_test_net_2_0", "corlib_plattest" };
+			string [] names = { "corlib_test_net_1_1", "corlib_test_net_2_0", "corlib_test_net_4_0", "corlib_plattest" };
 
 			foreach (string s in names)
 				if (Assembly.LoadWithPartialName (s) != null)
 					return;
-			Assertion.Fail ("Was not able to load any corlib test");
+			Assert.Fail ("Was not able to load any corlib test");
 		}
 
 #if !TARGET_JVM // GetObjectData currently not implemented for Assembly.
@@ -1088,21 +1089,26 @@ namespace MonoTests.System.Reflection
 		}
 #endif // TARGET_JVM
 
-#if NET_2_0
 		[Test]
 		public void ManifestModule ()
 		{
 			Assembly assembly = typeof (int).Assembly;
 			Module module = assembly.ManifestModule;
 			Assert.IsNotNull (module, "#1");
+
+#if NET_4_0
+			Assert.AreEqual ("MonoModule", module.GetType ().Name, "#2");
+#else
 			Assert.AreEqual (typeof (Module), module.GetType (), "#2");
+#endif
+
 			Assert.AreEqual ("mscorlib.dll", module.Name, "#3");
 			Assert.IsFalse (module.IsResource (), "#4");
 			Assert.IsTrue (assembly.GetModules ().Length > 0, "#5");
 			Assert.AreSame (module, assembly.GetModules () [0], "#6");
 			Assert.AreSame (module, assembly.ManifestModule, "#7");
 		}
-#endif
+
 
 		[Serializable ()]
 		private class AssemblyResolveHandler
@@ -1154,5 +1160,15 @@ namespace MonoTests.System.Reflection
 				Assert.AreEqual ("attributeType", ex.ParamName, "#6");
 			}
 		}
+
+		[Test]
+		public void GetTypeWithEmptyStringShouldThrow ()
+		{
+			try {
+				typeof (string).Assembly.GetType ("");
+				Assert.Fail ("#1");
+			} catch (ArgumentException) {}
+		}
+
 	}
 }
