@@ -2,37 +2,31 @@
  *
  * Copyright (c) Microsoft Corporation. 
  *
- * This source code is subject to terms and conditions of the Microsoft Public License. A 
+ * This source code is subject to terms and conditions of the Apache License, Version 2.0. A 
  * copy of the license can be found in the License.html file at the root of this distribution. If 
- * you cannot locate the  Microsoft Public License, please send an email to 
+ * you cannot locate the  Apache License, Version 2.0, please send an email to 
  * dlr@microsoft.com. By using this source code in any fashion, you are agreeing to be bound 
- * by the terms of the Microsoft Public License.
+ * by the terms of the Apache License, Version 2.0.
  *
  * You must not remove this notice, or any other, from this software.
  *
  *
  * ***************************************************************************/
-using System; using Microsoft;
 
-
-using System.Collections.Generic;
-using System.Diagnostics;
-#if CODEPLEX_40
-using System.Linq.Expressions;
+#if CLR2
+using Microsoft.Scripting.Ast;
 #else
-using Microsoft.Linq.Expressions;
+using System.Linq.Expressions;
 #endif
-using System.Reflection;
-
 #if SILVERLIGHT
 using System.Core;
 #endif
 
-#if CODEPLEX_40
+using System.Collections.Generic;
+using System.Diagnostics;
+using System.Reflection;
+
 namespace System.Dynamic.Utils {
-#else
-namespace Microsoft.Scripting.Utils {
-#endif
 
     internal static class TypeUtils {
         private const BindingFlags AnyStatic = BindingFlags.Static | BindingFlags.Public | BindingFlags.NonPublic;
@@ -155,7 +149,7 @@ namespace Microsoft.Scripting.Utils {
 
         internal static bool AreEquivalent(Type t1, Type t2)
         {
-#if MICROSOFT_SCRIPTING_CORE || SILVERLIGHT
+#if CLR2 || SILVERLIGHT
             return t1 == t2;
 #else
             return t1 == t2 || t1.IsEquivalentTo(t2);
@@ -254,6 +248,10 @@ namespace Microsoft.Scripting.Utils {
             if (source.IsInterface || dest.IsInterface) {
                 return true;
             }
+            // Variant delegate conversion
+            if (IsLegalExplicitVariantDelegateConversion(source, dest))
+                return true;
+                
             // Object conversion
             if (source == typeof(object) || dest == typeof(object)) {
                 return true;
@@ -261,6 +259,102 @@ namespace Microsoft.Scripting.Utils {
             return false;
         }
 
+        private static bool IsCovariant(Type t)
+        {
+            Debug.Assert(t != null);
+            return 0 != (t.GenericParameterAttributes & GenericParameterAttributes.Covariant);
+        }
+
+        private static bool IsContravariant(Type t)
+        {
+            Debug.Assert(t != null);
+            return 0 != (t.GenericParameterAttributes & GenericParameterAttributes.Contravariant);
+        }
+
+        private static bool IsInvariant(Type t)
+        {
+            Debug.Assert(t != null);
+            return 0 == (t.GenericParameterAttributes & GenericParameterAttributes.VarianceMask);
+        }
+
+        private static bool IsDelegate(Type t)
+        {
+            Debug.Assert(t != null);
+            return t.IsSubclassOf(typeof(System.MulticastDelegate));
+        }
+
+        internal static bool IsLegalExplicitVariantDelegateConversion(Type source, Type dest)
+        {
+            Debug.Assert(source != null && dest != null);
+
+            // There *might* be a legal conversion from a generic delegate type S to generic delegate type  T, 
+            // provided all of the follow are true:
+            //   o Both types are constructed generic types of the same generic delegate type, D<X1,... Xk>.
+            //     That is, S = D<S1...>, T = D<T1...>.
+            //   o If type parameter Xi is declared to be invariant then Si must be identical to Ti.
+            //   o If type parameter Xi is declared to be covariant ("out") then Si must be convertible
+            //     to Ti via an identify conversion,  implicit reference conversion, or explicit reference conversion.
+            //   o If type parameter Xi is declared to be contravariant ("in") then either Si must be identical to Ti, 
+            //     or Si and Ti must both be reference types.
+
+            if (!IsDelegate(source) || !IsDelegate(dest) || !source.IsGenericType || !dest.IsGenericType)
+                return false;
+
+            Type genericDelegate = source.GetGenericTypeDefinition();
+
+            if (dest.GetGenericTypeDefinition() != genericDelegate)
+                return false;
+
+            Type[] genericParameters = genericDelegate.GetGenericArguments();
+            Type[] sourceArguments = source.GetGenericArguments();
+            Type[] destArguments = dest.GetGenericArguments();
+
+            Debug.Assert(genericParameters != null);
+            Debug.Assert(sourceArguments != null);
+            Debug.Assert(destArguments != null);
+            Debug.Assert(genericParameters.Length == sourceArguments.Length);
+            Debug.Assert(genericParameters.Length == destArguments.Length);
+
+            for (int iParam = 0; iParam < genericParameters.Length; ++iParam)
+            {
+                Type sourceArgument = sourceArguments[iParam];
+                Type destArgument = destArguments[iParam];
+
+                Debug.Assert(sourceArgument != null && destArgument != null);
+               
+                // If the arguments are identical then this one is automatically good, so skip it.
+                if (AreEquivalent(sourceArgument, destArgument))
+                {
+                    continue;
+                }
+                
+                Type genericParameter = genericParameters[iParam];
+
+                Debug.Assert(genericParameter != null);
+
+                if (IsInvariant(genericParameter))
+                {
+                    return false;
+                }
+        
+                if (IsCovariant(genericParameter))
+                {
+                    if (!HasReferenceConversion(sourceArgument, destArgument))
+                    {
+                        return false;
+                    }
+                }
+                else if (IsContravariant(genericParameter))
+                {
+                    if (sourceArgument.IsValueType || destArgument.IsValueType)
+                    {
+                        return false;
+                    }
+                }
+            }
+            return true;
+        }
+
         internal static bool IsConvertible(Type type) {
             type = GetNonNullableType(type);
             if (type.IsEnum) {
